# coding=utf-8
"""Module taken from OpenStudio-standards.

https://github.com/NREL/openstudio-standards/blob/master/
lib/openstudio-standards/hvac/components/create.rb
"""
from __future__ import division

from honeybee_openstudio.openstudio import openstudio_model


def create_hx_air_to_air_sensible_and_latent(
        model, name=None, type=None, economizer_lockout=None,
        supply_air_outlet_temperature_control=None, frost_control_type=None,
        nominal_electric_power=None,
        sensible_heating_100_eff=None, latent_heating_100_eff=None,
        sensible_cooling_100_eff=None, latent_cooling_100_eff=None):
    """Creates HeatExchangerAirToAirSensibleAndLatent object."""
    hx = openstudio_model.HeatExchangerAirToAirSensibleAndLatent(model)
    if name is not None:
        hx.setName(name)

    if type is not None:
        hx.setHeatExchangerType(type)

    if frost_control_type is not None:
        hx.setFrostControlType(frost_control_type)

    if economizer_lockout is not None:
        hx.setEconomizerLockout(economizer_lockout)
    if supply_air_outlet_temperature_control is not None:
        hx.setSupplyAirOutletTemperatureControl(supply_air_outlet_temperature_control)
    if nominal_electric_power is not None:
        hx.setNominalElectricPower(nominal_electric_power)

    if sensible_heating_100_eff is not None:
        hx.setSensibleEffectivenessat100HeatingAirFlow(sensible_heating_100_eff)
    if latent_heating_100_eff is not None:
        hx.setLatentEffectivenessat100HeatingAirFlow(latent_heating_100_eff)
    if sensible_cooling_100_eff is not None:
        hx.setSensibleEffectivenessat100CoolingAirFlow(sensible_cooling_100_eff)
    if latent_cooling_100_eff is not None:
        hx.setLatentEffectivenessat100CoolingAirFlow(latent_cooling_100_eff)

    return hx



# coding=utf-8
"""Module taken from OpenStudio-standards.

https://github.com/NREL/openstudio-standards/blob/master/
lib/openstudio-standards/prototypes/common/objects/
Prototype.AirConditionerVariableRefrigerantFlow.rb
"""
from __future__ import division

from ladybug.datatype.temperature import Temperature

from honeybee_openstudio.openstudio import openstudio, openstudio_model
from .schedule import model_add_schedule

TEMPERATURE = Temperature()


def create_air_conditioner_variable_refrigerant_flow(
        model, name='VRF System', schedule=None, type=None,
        cooling_cop=4.287, heating_cop=4.147, heat_recovery=True,
        defrost_strategy='Resistive', condenser_type='AirCooled', condenser_loop=None,
        master_zone=None, priority_control_type='LoadPriority'):
    """Prototype AirConditionerVariableRefrigerantFlow object.

    Enters in default curves for coil by type of coil.

    Args:
        model: [OpenStudio::Model::Model] OpenStudio model object.
        name: [String] the name of the system, or None in which case it will be defaulted.
        schedule: [String] name of the availability schedule, or
            [<OpenStudio::Model::Schedule>] Schedule object, or None in which case
            default to always on.
        type: [String] the type of unit to reference for the correct curve set.
        cooling_cop: [Double] rated cooling coefficient of performance.
        heating_cop: [Double] rated heating coefficient of performance.
        heat_recovery: [Boolean] does the unit have heat recovery.
        defrost_strategy: [String] type of defrost strategy. Options are
            ReverseCycle or Resistive.
        condenser_type: [String] type of condenser.
            options are AirCooled (default), WaterCooled, and EvaporativelyCooled.
            if WaterCooled, the user most include a condenser_loop.
        master_zone: [<OpenStudio::Model::ThermalZone>] master control zone to
            switch between heating and cooling.
        priority_control_type: [String] Type of master thermostat priority
            control type. Options are LoadPriority, ZonePriority,
            ThermostatOffsetPriority, MasterThermostatPriority.
    """
    vrf_outdoor_unit = openstudio_model.AirConditionerVariableRefrigerantFlow(model)

    # set name
    name = 'VRF System' if name is None else name
    vrf_outdoor_unit.setName(name)

    # set availability schedule
    if schedule is None:  # default always on
        availability_schedule = model.alwaysOnDiscreteSchedule()
    elif isinstance(schedule, str):
        if schedule == 'alwaysOffDiscreteSchedule':
            availability_schedule = model.alwaysOffDiscreteSchedule()
        elif schedule == 'alwaysOnDiscreteSchedule':
            availability_schedule = model.alwaysOnDiscreteSchedule()
        else:
            availability_schedule = model_add_schedule(model, schedule)
    else:  # assume that it is an actual schedule object
        availability_schedule = schedule
    vrf_outdoor_unit.setAvailabilitySchedule(availability_schedule)

    # set cops
    if model.version() < openstudio.VersionString('2.9.0'):
        vrf_outdoor_unit.setRatedCoolingCOP(cooling_cop)
    else:
        vrf_outdoor_unit.setGrossRatedCoolingCOP(cooling_cop)
    vrf_outdoor_unit.setRatedHeatingCOP(heating_cop)

    # heat recovery
    vrf_outdoor_unit.setHeatPumpWasteHeatRecovery(heat_recovery)

    # defrost strategy
    vrf_outdoor_unit.setDefrostStrategy(defrost_strategy)

    # defaults
    vrf_outdoor_unit.setMinimumOutdoorTemperatureinCoolingMode(-15.0)
    vrf_outdoor_unit.setMaximumOutdoorTemperatureinCoolingMode(50.0)
    vrf_outdoor_unit.setMinimumOutdoorTemperatureinHeatingMode(-25.0)
    vrf_outdoor_unit.setMaximumOutdoorTemperatureinHeatingMode(16.1)
    vrf_outdoor_unit.setMinimumOutdoorTemperatureinHeatRecoveryMode(-10.0)
    vrf_outdoor_unit.setMaximumOutdoorTemperatureinHeatRecoveryMode(27.2)
    vrf_outdoor_unit.setEquivalentPipingLengthusedforPipingCorrectionFactorinCoolingMode(30.48)
    vrf_outdoor_unit.setEquivalentPipingLengthusedforPipingCorrectionFactorinHeatingMode(30.48)
    vrf_outdoor_unit.setVerticalHeightusedforPipingCorrectionFactor(10.668)

    # condenser type
    if condenser_type == 'WaterCooled':
        vrf_outdoor_unit.setString(56, condenser_type)
        # require condenser_loop
        if not condenser_loop:
            msg = 'Must specify condenser_loop for vrf_outdoor_unit if WaterCooled'
            raise ValueError(msg)
        condenser_loop.addDemandBranchForComponent(vrf_outdoor_unit)
    elif condenser_type == 'EvaporativelyCooled':
        vrf_outdoor_unit.setString(56, condenser_type)

    # set master zone
    if master_zone is not None and master_zone.to_ThermalZone().is_initialized():
        vrf_outdoor_unit.setZoneforMasterThermostatLocation(master_zone)
        vrf_outdoor_unit.setMasterThermostatPriorityControlType(priority_control_type)

    vrf_cool_cap_f_of_low_temp = None
    vrf_cool_cap_ratio_boundary = None
    vrf_cool_cap_f_of_high_temp = None
    vrf_cool_eir_f_of_low_temp = None
    vrf_cool_eir_ratio_boundary = None
    vrf_cool_eir_f_of_high_temp = None
    vrf_cooling_eir_low_plr = None
    vrf_cooling_eir_high_plr = None
    vrf_cooling_comb_ratio = None
    vrf_cooling_cplffplr = None
    vrf_heat_cap_f_of_low_temp = None
    vrf_heat_cap_ratio_boundary = None
    vrf_heat_cap_f_of_high_temp = None
    vrf_heat_eir_f_of_low_temp = None
    vrf_heat_eir_boundary = None
    vrf_heat_eir_f_of_high_temp = None
    vrf_heating_eir_low_plr = None
    vrf_heating_eir_hi_plr = None
    vrf_heating_comb_ratio = None
    vrf_heating_cplffplr = None
    vrf_defrost_eir_f_of_temp = None

    # curve sets
    if type == 'OS default':
        pass  # use OS default curves

    else:  # default curve set
        # based on DAIKINREYQ 120 on BCL

        # Cooling Capacity Ratio Modifier Function of Low Temperature Curve
        vrf_cool_cap_f_of_low_temp = openstudio_model.CurveBiquadratic(model)
        vrf_cool_cap_f_of_low_temp.setName('vrf_cool_cap_f_of_low_temp')
        vrf_cool_cap_f_of_low_temp.setCoefficient1Constant(-1.69653019339465)
        vrf_cool_cap_f_of_low_temp.setCoefficient2x(0.207248180531939)
        vrf_cool_cap_f_of_low_temp.setCoefficient3xPOW2(-0.00343146229659024)
        vrf_cool_cap_f_of_low_temp.setCoefficient4y(0.016381597419714)
        vrf_cool_cap_f_of_low_temp.setCoefficient5yPOW2(-6.7387172629965e-05)
        vrf_cool_cap_f_of_low_temp.setCoefficient6xTIMESY(-0.000849848402870241)
        vrf_cool_cap_f_of_low_temp.setMinimumValueofx(13.9)
        vrf_cool_cap_f_of_low_temp.setMaximumValueofx(23.9)
        vrf_cool_cap_f_of_low_temp.setMinimumValueofy(-5.0)
        vrf_cool_cap_f_of_low_temp.setMaximumValueofy(43.3)
        vrf_cool_cap_f_of_low_temp.setMinimumCurveOutput(0.59)
        vrf_cool_cap_f_of_low_temp.setMaximumCurveOutput(1.33)

        # Cooling Capacity Ratio Boundary Curve
        vrf_cool_cap_ratio_boundary = openstudio_model.CurveCubic(model)
        vrf_cool_cap_ratio_boundary.setName('vrf_cool_cap_ratio_boundary')
        vrf_cool_cap_ratio_boundary.setCoefficient1Constant(25.73)
        vrf_cool_cap_ratio_boundary.setCoefficient2x(-0.03150043)
        vrf_cool_cap_ratio_boundary.setCoefficient3xPOW2(-0.01416595)
        vrf_cool_cap_ratio_boundary.setCoefficient4xPOW3(0.0)
        vrf_cool_cap_ratio_boundary.setMinimumValueofx(11.0)
        vrf_cool_cap_ratio_boundary.setMaximumValueofx(30.0)

        # Cooling Capacity Ratio Modifier Function of High Temperature Curve
        vrf_cool_cap_f_of_high_temp = openstudio_model.CurveBiquadratic(model)
        vrf_cool_cap_f_of_high_temp.setName('vrf_cool_cap_f_of_high_temp')
        vrf_cool_cap_f_of_high_temp.setCoefficient1Constant(0.6867358)
        vrf_cool_cap_f_of_high_temp.setCoefficient2x(0.0207631)
        vrf_cool_cap_f_of_high_temp.setCoefficient3xPOW2(0.0005447)
        vrf_cool_cap_f_of_high_temp.setCoefficient4y(-0.0016218)
        vrf_cool_cap_f_of_high_temp.setCoefficient5yPOW2(-4.259e-07)
        vrf_cool_cap_f_of_high_temp.setCoefficient6xTIMESY(-0.0003392)
        vrf_cool_cap_f_of_high_temp.setMinimumValueofx(15.0)
        vrf_cool_cap_f_of_high_temp.setMaximumValueofx(24.0)
        vrf_cool_cap_f_of_high_temp.setMinimumValueofy(16.0)
        vrf_cool_cap_f_of_high_temp.setMaximumValueofy(43.0)

        # Cooling Energy Input Ratio Modifier Function of Low Temperature Curve
        vrf_cool_eir_f_of_low_temp = openstudio_model.CurveBiquadratic(model)
        vrf_cool_eir_f_of_low_temp.setName('vrf_cool_eir_f_of_low_temp')
        vrf_cool_eir_f_of_low_temp.setCoefficient1Constant(-1.61908214818635)
        vrf_cool_eir_f_of_low_temp.setCoefficient2x(0.185964818731756)
        vrf_cool_eir_f_of_low_temp.setCoefficient3xPOW2(-0.00389610393381592)
        vrf_cool_eir_f_of_low_temp.setCoefficient4y(-0.00901995326324613)
        vrf_cool_eir_f_of_low_temp.setCoefficient5yPOW2(0.00030340007815629)
        vrf_cool_eir_f_of_low_temp.setCoefficient6xTIMESY(0.000476048529099348)
        vrf_cool_eir_f_of_low_temp.setMinimumValueofx(13.9)
        vrf_cool_eir_f_of_low_temp.setMaximumValueofx(23.9)
        vrf_cool_eir_f_of_low_temp.setMinimumValueofy(-5.0)
        vrf_cool_eir_f_of_low_temp.setMaximumValueofy(43.3)
        vrf_cool_eir_f_of_low_temp.setMinimumCurveOutput(0.27)
        vrf_cool_eir_f_of_low_temp.setMaximumCurveOutput(1.15)

        # Cooling Energy Input Ratio Boundary Curve
        vrf_cool_eir_ratio_boundary = openstudio_model.CurveCubic(model)
        vrf_cool_eir_ratio_boundary.setName('vrf_cool_eir_ratio_boundary')
        vrf_cool_eir_ratio_boundary.setCoefficient1Constant(25.73473775)
        vrf_cool_eir_ratio_boundary.setCoefficient2x(-0.03150043)
        vrf_cool_eir_ratio_boundary.setCoefficient3xPOW2(-0.01416595)
        vrf_cool_eir_ratio_boundary.setCoefficient4xPOW3(0.0)
        vrf_cool_eir_ratio_boundary.setMinimumValueofx(15.0)
        vrf_cool_eir_ratio_boundary.setMaximumValueofx(24.0)

        # Cooling Energy Input Ratio Modifier Function of High Temperature Curve
        vrf_cool_eir_f_of_high_temp = openstudio_model.CurveBiquadratic(model)
        vrf_cool_eir_f_of_high_temp.setName('vrf_cool_eir_f_of_high_temp')
        vrf_cool_eir_f_of_high_temp.setCoefficient1Constant(-1.4395110176)
        vrf_cool_eir_f_of_high_temp.setCoefficient2x(0.1619850459)
        vrf_cool_eir_f_of_high_temp.setCoefficient3xPOW2(-0.0034911781)
        vrf_cool_eir_f_of_high_temp.setCoefficient4y(0.0269442645)
        vrf_cool_eir_f_of_high_temp.setCoefficient5yPOW2(0.0001346163)
        vrf_cool_eir_f_of_high_temp.setCoefficient6xTIMESY(-0.0006714941)
        vrf_cool_eir_f_of_high_temp.setMinimumValueofx(15.0)
        vrf_cool_eir_f_of_high_temp.setMaximumValueofx(23.9)
        vrf_cool_eir_f_of_high_temp.setMinimumValueofy(16.8)
        vrf_cool_eir_f_of_high_temp.setMaximumValueofy(43.3)

        # Cooling Energy Input Ratio Modifier Function of Low Part-Load Ratio Curve
        vrf_cooling_eir_low_plr = openstudio_model.CurveCubic(model)
        vrf_cooling_eir_low_plr.setName('vrf_cool_eir_f_of_low_temp')
        vrf_cooling_eir_low_plr.setCoefficient1Constant(0.0734992169827752)
        vrf_cooling_eir_low_plr.setCoefficient2x(0.334783365234032)
        vrf_cooling_eir_low_plr.setCoefficient3xPOW2(0.591613015486343)
        vrf_cooling_eir_low_plr.setCoefficient4xPOW3(0.0)
        vrf_cooling_eir_low_plr.setMinimumValueofx(0.25)
        vrf_cooling_eir_low_plr.setMaximumValueofx(1.0)
        vrf_cooling_eir_low_plr.setMinimumCurveOutput(0.0)
        vrf_cooling_eir_low_plr.setMaximumCurveOutput(1.0)

        # Cooling Energy Input Ratio Modifier Function of High Part-Load Ratio Curve
        vrf_cooling_eir_high_plr = openstudio_model.CurveCubic(model)
        vrf_cooling_eir_high_plr.setName('vrf_cooling_eir_high_plr')
        vrf_cooling_eir_high_plr.setCoefficient1Constant(1.0)
        vrf_cooling_eir_high_plr.setCoefficient2x(0.0)
        vrf_cooling_eir_high_plr.setCoefficient3xPOW2(0.0)
        vrf_cooling_eir_high_plr.setCoefficient4xPOW3(0.0)
        vrf_cooling_eir_high_plr.setMinimumValueofx(1.0)
        vrf_cooling_eir_high_plr.setMaximumValueofx(1.5)

        # Cooling Combination Ratio Correction Factor Curve
        vrf_cooling_comb_ratio = openstudio_model.CurveCubic(model)
        vrf_cooling_comb_ratio.setName('vrf_cooling_comb_ratio')
        vrf_cooling_comb_ratio.setCoefficient1Constant(0.24034)
        vrf_cooling_comb_ratio.setCoefficient2x(-0.21873)
        vrf_cooling_comb_ratio.setCoefficient3xPOW2(1.97941)
        vrf_cooling_comb_ratio.setCoefficient4xPOW3(-1.02636)
        vrf_cooling_comb_ratio.setMinimumValueofx(0.5)
        vrf_cooling_comb_ratio.setMaximumValueofx(2.0)
        vrf_cooling_comb_ratio.setMinimumCurveOutput(0.5)
        vrf_cooling_comb_ratio.setMaximumCurveOutput(1.056)

        # Cooling Part-Load Fraction Correlation Curve
        vrf_cooling_cplffplr = openstudio_model.CurveCubic(model)
        vrf_cooling_cplffplr.setName('vrf_cooling_cplffplr')
        vrf_cooling_cplffplr.setCoefficient1Constant(0.85)
        vrf_cooling_cplffplr.setCoefficient2x(0.15)
        vrf_cooling_cplffplr.setCoefficient3xPOW2(0.0)
        vrf_cooling_cplffplr.setCoefficient4xPOW3(0.0)
        vrf_cooling_cplffplr.setMinimumValueofx(1.0)
        vrf_cooling_cplffplr.setMaximumValueofx(1.0)

        # Heating Capacity Ratio Modifier Function of Low Temperature Curve Name
        vrf_heat_cap_f_of_low_temp = openstudio_model.CurveBiquadratic(model)
        vrf_heat_cap_f_of_low_temp.setName('vrf_heat_cap_f_of_low_temp')
        vrf_heat_cap_f_of_low_temp.setCoefficient1Constant(0.983220174655636)
        vrf_heat_cap_f_of_low_temp.setCoefficient2x(0.0157167577703294)
        vrf_heat_cap_f_of_low_temp.setCoefficient3xPOW2(-0.000835032422884084)
        vrf_heat_cap_f_of_low_temp.setCoefficient4y(0.0522939264581759)
        vrf_heat_cap_f_of_low_temp.setCoefficient5yPOW2(-0.000531556035364549)
        vrf_heat_cap_f_of_low_temp.setCoefficient6xTIMESY(-0.00190605953116024)
        vrf_heat_cap_f_of_low_temp.setMinimumValueofx(16.1)
        vrf_heat_cap_f_of_low_temp.setMaximumValueofx(23.9)
        vrf_heat_cap_f_of_low_temp.setMinimumValueofy(-25.0)
        vrf_heat_cap_f_of_low_temp.setMaximumValueofy(13.3)
        vrf_heat_cap_f_of_low_temp.setMinimumCurveOutput(0.515151515151515)
        vrf_heat_cap_f_of_low_temp.setMaximumCurveOutput(1.2)

        # Heating Capacity Ratio Boundary Curve Name
        vrf_heat_cap_ratio_boundary = openstudio_model.CurveCubic(model)
        vrf_heat_cap_ratio_boundary.setName('vrf_heat_cap_ratio_boundary')
        vrf_heat_cap_ratio_boundary.setCoefficient1Constant(58.577)
        vrf_heat_cap_ratio_boundary.setCoefficient2x(-3.0255)
        vrf_heat_cap_ratio_boundary.setCoefficient3xPOW2(0.0193)
        vrf_heat_cap_ratio_boundary.setCoefficient4xPOW3(0.0)
        vrf_heat_cap_ratio_boundary.setMinimumValueofx(15)
        vrf_heat_cap_ratio_boundary.setMaximumValueofx(23.9)

        # Heating Capacity Ratio Modifier Function of High Temperature Curve Name
        vrf_heat_cap_f_of_high_temp = openstudio_model.CurveBiquadratic(model)
        vrf_heat_cap_f_of_high_temp.setName('vrf_heat_cap_f_of_high_temp')
        vrf_heat_cap_f_of_high_temp.setCoefficient1Constant(2.5859872368)
        vrf_heat_cap_f_of_high_temp.setCoefficient2x(-0.0953227101)
        vrf_heat_cap_f_of_high_temp.setCoefficient3xPOW2(0.0009553288)
        vrf_heat_cap_f_of_high_temp.setCoefficient4y(0.0)
        vrf_heat_cap_f_of_high_temp.setCoefficient5yPOW2(0.0)
        vrf_heat_cap_f_of_high_temp.setCoefficient6xTIMESY(0.0)
        vrf_heat_cap_f_of_high_temp.setMinimumValueofx(21.1)
        vrf_heat_cap_f_of_high_temp.setMaximumValueofx(27.2)
        vrf_heat_cap_f_of_high_temp.setMinimumValueofy(-944)
        vrf_heat_cap_f_of_high_temp.setMaximumValueofy(15)

        # Heating Energy Input Ratio Modifier Function of Low Temperature Curve Name
        vrf_heat_eir_f_of_low_temp = openstudio_model.CurveBiquadratic(model)
        vrf_heat_eir_f_of_low_temp.setName('vrf_heat_eir_f_of_low_temp')
        vrf_heat_eir_f_of_low_temp.setCoefficient1Constant(0.756830029796909)
        vrf_heat_eir_f_of_low_temp.setCoefficient2x(0.0457499799042671)
        vrf_heat_eir_f_of_low_temp.setCoefficient3xPOW2(-0.00136357240431388)
        vrf_heat_eir_f_of_low_temp.setCoefficient4y(0.0554884599902023)
        vrf_heat_eir_f_of_low_temp.setCoefficient5yPOW2(-0.00120700875497686)
        vrf_heat_eir_f_of_low_temp.setCoefficient6xTIMESY(-0.00303329271420931)
        vrf_heat_eir_f_of_low_temp.setMinimumValueofx(16.1)
        vrf_heat_eir_f_of_low_temp.setMaximumValueofx(23.9)
        vrf_heat_eir_f_of_low_temp.setMinimumValueofy(-25.0)
        vrf_heat_eir_f_of_low_temp.setMaximumValueofy(13.3)
        vrf_heat_eir_f_of_low_temp.setMinimumCurveOutput(0.7)
        vrf_heat_eir_f_of_low_temp.setMaximumCurveOutput(1.184)

        # Heating Energy Input Ratio Boundary Curve Name
        vrf_heat_eir_boundary = openstudio_model.CurveCubic(model)
        vrf_heat_eir_boundary.setName('vrf_heat_eir_boundary')
        vrf_heat_eir_boundary.setCoefficient1Constant(58.577)
        vrf_heat_eir_boundary.setCoefficient2x(-3.0255)
        vrf_heat_eir_boundary.setCoefficient3xPOW2(0.0193)
        vrf_heat_eir_boundary.setCoefficient4xPOW3(0.0)
        vrf_heat_eir_boundary.setMinimumValueofx(15.0)
        vrf_heat_eir_boundary.setMaximumValueofx(23.9)

        # Heating Energy Input Ratio Modifier Function of High Temperature Curve Name
        vrf_heat_eir_f_of_high_temp = openstudio_model.CurveBiquadratic(model)
        vrf_heat_eir_f_of_high_temp.setName('vrf_heat_eir_f_of_high_temp')
        vrf_heat_eir_f_of_high_temp.setCoefficient1Constant(1.3885703646)
        vrf_heat_eir_f_of_high_temp.setCoefficient2x(-0.0229771462)
        vrf_heat_eir_f_of_high_temp.setCoefficient3xPOW2(0.000537274)
        vrf_heat_eir_f_of_high_temp.setCoefficient4y(-0.0273936962)
        vrf_heat_eir_f_of_high_temp.setCoefficient5yPOW2(0.0004030426)
        vrf_heat_eir_f_of_high_temp.setCoefficient6xTIMESY(-5.9786e-05)
        vrf_heat_eir_f_of_high_temp.setMinimumValueofx(21.1)
        vrf_heat_eir_f_of_high_temp.setMaximumValueofx(27.2)
        vrf_heat_eir_f_of_high_temp.setMinimumValueofy(0.0)
        vrf_heat_eir_f_of_high_temp.setMaximumValueofy(1.0)

        # Heating Performance Curve Outdoor Temperature Type
        vrf_outdoor_unit.setHeatingPerformanceCurveOutdoorTemperatureType(
            'WetBulbTemperature')

        # Heating Energy Input Ratio Modifier Function of Low Part-Load Ratio Curve Name
        vrf_heating_eir_low_plr = openstudio_model.CurveCubic(model)
        vrf_heating_eir_low_plr.setName('vrf_heating_eir_low_plr')
        vrf_heating_eir_low_plr.setCoefficient1Constant(0.0724906507105475)
        vrf_heating_eir_low_plr.setCoefficient2x(0.658189977561701)
        vrf_heating_eir_low_plr.setCoefficient3xPOW2(0.269259536275246)
        vrf_heating_eir_low_plr.setCoefficient4xPOW3(0.0)
        vrf_heating_eir_low_plr.setMinimumValueofx(0.25)
        vrf_heating_eir_low_plr.setMaximumValueofx(1.0)
        vrf_heating_eir_low_plr.setMinimumCurveOutput(0.0)
        vrf_heating_eir_low_plr.setMaximumCurveOutput(1.0)

        # Heating Energy Input Ratio Modifier Function of High Part-Load Ratio Curve Name
        vrf_heating_eir_hi_plr = openstudio_model.CurveCubic(model)
        vrf_heating_eir_hi_plr.setName('vrf_heating_eir_hi_plr')
        vrf_heating_eir_hi_plr.setCoefficient1Constant(1.0)
        vrf_heating_eir_hi_plr.setCoefficient2x(0.0)
        vrf_heating_eir_hi_plr.setCoefficient3xPOW2(0.0)
        vrf_heating_eir_hi_plr.setCoefficient4xPOW3(0.0)
        vrf_heating_eir_hi_plr.setMinimumValueofx(1.0)
        vrf_heating_eir_hi_plr.setMaximumValueofx(1.5)

        # Heating Combination Ratio Correction Factor Curve Name
        vrf_heating_comb_ratio = openstudio_model.CurveCubic(model)
        vrf_heating_comb_ratio.setName('vrf_heating_comb_ratio')
        vrf_heating_comb_ratio.setCoefficient1Constant(0.62115)
        vrf_heating_comb_ratio.setCoefficient2x(-1.55798)
        vrf_heating_comb_ratio.setCoefficient3xPOW2(3.36817)
        vrf_heating_comb_ratio.setCoefficient4xPOW3(-1.4224)
        vrf_heating_comb_ratio.setMinimumValueofx(0.5)
        vrf_heating_comb_ratio.setMaximumValueofx(2.0)
        vrf_heating_comb_ratio.setMinimumCurveOutput(0.5)
        vrf_heating_comb_ratio.setMaximumCurveOutput(1.155)

        # Heating Part-Load Fraction Correlation Curve Name
        vrf_heating_cplffplr = openstudio_model.CurveCubic(model)
        vrf_heating_cplffplr.setName('vrf_heating_cplffplr')
        vrf_heating_cplffplr.setCoefficient1Constant(0.85)
        vrf_heating_cplffplr.setCoefficient2x(0.15)
        vrf_heating_cplffplr.setCoefficient3xPOW2(0.0)
        vrf_heating_cplffplr.setCoefficient4xPOW3(0.0)
        vrf_heating_cplffplr.setMinimumValueofx(1.0)
        vrf_heating_cplffplr.setMaximumValueofx(1.0)

        # Defrost Energy Input Ratio Modifier Function of Temperature Curve
        vrf_defrost_eir_f_of_temp = openstudio_model.CurveBiquadratic(model)
        vrf_defrost_eir_f_of_temp.setName('vrf_defrost_eir_f_of_temp')
        vrf_defrost_eir_f_of_temp.setCoefficient1Constant(-1.61908214818635)
        vrf_defrost_eir_f_of_temp.setCoefficient2x(0.185964818731756)
        vrf_defrost_eir_f_of_temp.setCoefficient3xPOW2(-0.00389610393381592)
        vrf_defrost_eir_f_of_temp.setCoefficient4y(-0.00901995326324613)
        vrf_defrost_eir_f_of_temp.setCoefficient5yPOW2(0.00030340007815629)
        vrf_defrost_eir_f_of_temp.setCoefficient6xTIMESY(0.000476048529099348)
        vrf_defrost_eir_f_of_temp.setMinimumValueofx(13.9)
        vrf_defrost_eir_f_of_temp.setMaximumValueofx(23.9)
        vrf_defrost_eir_f_of_temp.setMinimumValueofy(-5.0)
        vrf_defrost_eir_f_of_temp.setMaximumValueofy(50.0)
        vrf_defrost_eir_f_of_temp.setMinimumCurveOutput(0.27)
        vrf_defrost_eir_f_of_temp.setMaximumCurveOutput(1.155)

        # set defrost control
        vrf_outdoor_unit.setDefrostStrategy('ReverseCycle')
        vrf_outdoor_unit.setDefrostControl('OnDemand')

    if vrf_cool_cap_f_of_low_temp is not None:
        vrf_outdoor_unit.setCoolingCapacityRatioModifierFunctionofLowTemperatureCurve(
            vrf_cool_cap_f_of_low_temp)
    if vrf_cool_cap_ratio_boundary is not None:
        vrf_outdoor_unit.setCoolingCapacityRatioBoundaryCurve(
            vrf_cool_cap_ratio_boundary)
    if vrf_cool_cap_f_of_high_temp is not None:
        vrf_outdoor_unit.setCoolingCapacityRatioModifierFunctionofHighTemperatureCurve(
            vrf_cool_cap_f_of_high_temp)
    if vrf_cool_eir_f_of_low_temp is not None:
        vrf_outdoor_unit.setCoolingEnergyInputRatioModifierFunctionofLowTemperatureCurve(
            vrf_cool_eir_f_of_low_temp)
    if vrf_cool_eir_ratio_boundary is not None:
        vrf_outdoor_unit.setCoolingEnergyInputRatioBoundaryCurve(
            vrf_cool_eir_ratio_boundary)
    if vrf_cool_eir_f_of_high_temp is not None:
        vrf_outdoor_unit.setCoolingEnergyInputRatioModifierFunctionofHighTemperatureCurve(
            vrf_cool_eir_f_of_high_temp)
    if vrf_cooling_eir_low_plr is not None:
        vrf_outdoor_unit.setCoolingEnergyInputRatioModifierFunctionofLowPartLoadRatioCurve(
            vrf_cooling_eir_low_plr)
    if vrf_cooling_eir_high_plr is not None:
        vrf_outdoor_unit.setCoolingEnergyInputRatioModifierFunctionofHighPartLoadRatioCurve(
            vrf_cooling_eir_high_plr)
    if vrf_cooling_comb_ratio is not None:
        vrf_outdoor_unit.setCoolingCombinationRatioCorrectionFactorCurve(
            vrf_cooling_comb_ratio)
    if vrf_cooling_cplffplr is not None:
        vrf_outdoor_unit.setCoolingPartLoadFractionCorrelationCurve(
            vrf_cooling_cplffplr)
    if vrf_heat_cap_f_of_low_temp is not None:
        vrf_outdoor_unit.setHeatingCapacityRatioModifierFunctionofLowTemperatureCurve(
            vrf_heat_cap_f_of_low_temp)
    if vrf_heat_cap_ratio_boundary is not None:
        vrf_outdoor_unit.setHeatingCapacityRatioBoundaryCurve(
            vrf_heat_cap_ratio_boundary)
    if vrf_heat_cap_f_of_high_temp is not None:
        vrf_outdoor_unit.setHeatingCapacityRatioModifierFunctionofHighTemperatureCurve(
            vrf_heat_cap_f_of_high_temp)
    if vrf_heat_eir_f_of_low_temp is not None:
        vrf_outdoor_unit.setHeatingEnergyInputRatioModifierFunctionofLowTemperatureCurve(
            vrf_heat_eir_f_of_low_temp)
    if vrf_heat_eir_boundary is not None:
        vrf_outdoor_unit.setHeatingEnergyInputRatioBoundaryCurve(vrf_heat_eir_boundary)
    if vrf_heat_eir_f_of_high_temp is not None:
        vrf_outdoor_unit.setHeatingEnergyInputRatioModifierFunctionofHighTemperatureCurve(
            vrf_heat_eir_f_of_high_temp)
    if vrf_heating_eir_low_plr is not None:
        vrf_outdoor_unit.setHeatingEnergyInputRatioModifierFunctionofLowPartLoadRatioCurve(
            vrf_heating_eir_low_plr)
    if vrf_heating_eir_hi_plr is not None:
        vrf_outdoor_unit.setHeatingEnergyInputRatioModifierFunctionofHighPartLoadRatioCurve(
            vrf_heating_eir_hi_plr)
    if vrf_heating_comb_ratio is not None:
        vrf_outdoor_unit.setHeatingCombinationRatioCorrectionFactorCurve(
            vrf_heating_comb_ratio)
    if vrf_heating_cplffplr is not None:
        vrf_outdoor_unit.setHeatingPartLoadFractionCorrelationCurve(vrf_heating_cplffplr)
    if vrf_defrost_eir_f_of_temp is not None:
        vrf_outdoor_unit.setDefrostEnergyInputRatioModifierFunctionofTemperatureCurve(
            vrf_defrost_eir_f_of_temp)

    return vrf_outdoor_unit



# coding=utf-8
"""OpenStudio load translators."""
from __future__ import division

from ladybug_geometry.geometry3d import Point3D
from honeybee.altnumber import autocalculate
from honeybee.typing import clean_ep_string
from honeybee_energy.schedule.ruleset import ScheduleRuleset
from honeybee_energy.lib.scheduletypelimits import temperature
from honeybee_energy.lib.schedules import always_on
from honeybee_energy.load.people import People
from honeybee_energy.load.lighting import Lighting
from honeybee_energy.load.equipment import ElectricEquipment, GasEquipment
from honeybee_energy.load.process import Process
from honeybee_energy.load.hotwater import ServiceHotWater
from honeybee_energy.load.infiltration import Infiltration
from honeybee_energy.load.ventilation import Ventilation
from honeybee_energy.load.setpoint import Setpoint
from honeybee_energy.load.daylight import DaylightingControl

from honeybee_openstudio.openstudio import OSPeopleDefinition, OSPeople, \
    OSLightsDefinition, OSLights, OSElectricEquipmentDefinition, OSElectricEquipment, \
    OSGasEquipmentDefinition, OSGasEquipment, OSOtherEquipmentDefinition, \
    OSOtherEquipment, OSWaterUseEquipmentDefinition, OSWaterUseEquipment, \
    OSWaterUseConnections, OSSpaceInfiltrationDesignFlowRate, \
    OSDesignSpecificationOutdoorAir, OSThermostatSetpointDualSetpoint, \
    OSZoneControlHumidistat, OSDaylightingControl, OSScheduleRuleset, os_vector_len


def _create_constant_schedule(schedule_name, schedule_value, os_model):
    """Utility function for creating a schedule with a single value."""
    # check if a constant schedule already exists and, if not, create it
    exist_schedule = os_model.getScheduleByName(schedule_name)
    if exist_schedule.is_initialized():  # return the existing schedule
        return exist_schedule.get()
    else:  # create the schedule
        os_sch = OSScheduleRuleset(os_model, schedule_value)
        os_sch.setName(schedule_name)
        return os_sch


"""____________TRANSLATORS TO OPENSTUDIO____________"""


def people_to_openstudio(people, os_model):
    """Convert Honeybee People object to OpenStudio People object."""
    # create people OpenStudio object and set identifier
    os_people_def = OSPeopleDefinition(os_model)
    os_people = OSPeople(os_people_def)
    os_people_def.setName(people.identifier)
    os_people.setName(people.identifier)
    if people._display_name is not None:
        os_people_def.setDisplayName(people.display_name)
        os_people.setDisplayName(people.display_name)
    # assign people per space floor area
    os_people_def.setPeopleperSpaceFloorArea(people.people_per_area)
    # assign occupancy schedule
    occupancy_sch = os_model.getScheduleByName(people.occupancy_schedule.identifier)
    if occupancy_sch.is_initialized():
        occupancy_sch = occupancy_sch.get()
        os_people.setNumberofPeopleSchedule(occupancy_sch)
    # assign activity schedule
    activity_sch = os_model.getScheduleByName(people.activity_schedule.identifier)
    if activity_sch.is_initialized():
        activity_sch = activity_sch.get()
        os_people.setActivityLevelSchedule(activity_sch)
    # assign radiant and latent fractions
    os_people_def.setFractionRadiant(people.radiant_fraction)
    if people.latent_fraction == autocalculate:
        os_people_def.autocalculateSensibleHeatFraction()
    else:
        os_people_def.setSensibleHeatFraction(1.0 - float(people.latent_fraction))
    return os_people


def lighting_to_openstudio(lighting, os_model):
    """Convert Honeybee Lighting object to OpenStudio Lights object."""
    # create people OpenStudio object and set identifier
    os_lighting_def = OSLightsDefinition(os_model)
    os_lighting = OSLights(os_lighting_def)
    os_lighting_def.setName(lighting.identifier)
    os_lighting.setName(lighting.identifier)
    if lighting._display_name is not None:
        os_lighting_def.setDisplayName(lighting.display_name)
        os_lighting.setDisplayName(lighting.display_name)
    # assign watts per space floor area
    os_lighting_def.setWattsperSpaceFloorArea(lighting.watts_per_area)
    # assign lighting schedule
    lighting_schedule = os_model.getScheduleByName(lighting.schedule.identifier)
    if lighting_schedule.is_initialized():
        lighting_schedule = lighting_schedule.get()
        os_lighting.setSchedule(lighting_schedule)
    # assign visible, radiant, and return air fractions
    os_lighting_def.setFractionVisible(lighting.visible_fraction)
    os_lighting_def.setFractionRadiant(lighting.radiant_fraction)
    os_lighting_def.setReturnAirFraction(lighting.return_air_fraction)
    return os_lighting


def _equipment_to_openstudio(equipment, os_equip_def, os_equip, os_model):
    """Process any type of equipment object to OpenStudio."""
    os_equip_def.setName(equipment.identifier)
    os_equip.setName(equipment.identifier)
    if equipment._display_name is not None:
        os_equip_def.setDisplayName(equipment.display_name)
        os_equip.setDisplayName(equipment.display_name)
    # assign schedule
    equip_schedule = os_model.getScheduleByName(equipment.schedule.identifier)
    if equip_schedule.is_initialized():
        equip_schedule = equip_schedule.get()
        os_equip.setSchedule(equip_schedule)
    # assign radiant, latent, and lost fractions
    os_equip_def.setFractionRadiant(equipment.radiant_fraction)
    os_equip_def.setFractionLatent(equipment.latent_fraction)
    os_equip_def.setFractionLost(equipment.lost_fraction)


def electric_equipment_to_openstudio(equipment, os_model):
    """Convert Honeybee ElectricEquipment object to OpenStudio ElectricEquipment object.
    """
    # create the OpenStudio object
    os_equip_def = OSElectricEquipmentDefinition(os_model)
    os_equip = OSElectricEquipment(os_equip_def)
    _equipment_to_openstudio(equipment, os_equip_def, os_equip, os_model)
    # assign watts per area
    os_equip_def.setWattsperSpaceFloorArea(equipment.watts_per_area)
    # ensure that it's always reported under electric equipment
    os_equip.setEndUseSubcategory('Electric Equipment')
    return os_equip


def gas_equipment_to_openstudio(equipment, os_model):
    """Convert Honeybee GasEquipment object to OpenStudio GasEquipment object.
    """
    # create the OpenStudio object
    os_equip_def = OSGasEquipmentDefinition(os_model)
    os_equip = OSGasEquipment(os_equip_def)
    _equipment_to_openstudio(equipment, os_equip_def, os_equip, os_model)
    # assign watts per area
    os_equip_def.setWattsperSpaceFloorArea(equipment.watts_per_area)
    # ensure that it's always reported under electric equipment
    os_equip.setEndUseSubcategory('Electric Equipment')
    return os_equip


def process_to_openstudio(process, os_model):
    """Convert Honeybee Process object to OpenStudio OtherEquipment object.
    """
    # create the OpenStudio object
    os_equip_def = OSOtherEquipmentDefinition(os_model)
    os_equip = OSOtherEquipment(os_equip_def)
    _equipment_to_openstudio(process, os_equip_def, os_equip, os_model)
    # assign watts
    os_equip_def.setDesignLevel(process.watts)
    # assign the fuel type and end use category
    os_equip.setFuelType(process.fuel_type)
    os_equip.setEndUseSubcategory(process.end_use_category)
    return os_equip


def hot_water_to_openstudio(hot_water, room, os_model):
    """Convert Honeybee ServiceHotWater to OpenStudio WaterUseConnections.

    Args:
        hot_water: The Honeybee-energy ServiceHotWater object to be translated
            to OpenStudio.
        room: The Honeybee Room object to which the ServiceHotWater is assigned.
            This is required in order to compute the total flow rate from the
            floor area.
        os_model: The OpenStudio Model to which the WaterUseEquipment and
            WaterUseConnections objects will be added.
    """
    # create water use equipment + connection and set identifier
    os_shw_def = OSWaterUseEquipmentDefinition(os_model)
    os_shw = OSWaterUseEquipment(os_shw_def)
    os_shw_conn = OSWaterUseConnections(os_model)
    os_shw_conn.addWaterUseEquipment(os_shw)
    u_id = '{}..{}'.format(hot_water.identifier, room.identifier)
    os_shw_def.setName(u_id)
    os_shw.setName(u_id)
    os_shw_conn.setName(u_id)
    if hot_water._display_name is not None:
        os_shw_def.setDisplayName(hot_water.display_name)
        os_shw.setDisplayName(hot_water.display_name)
        os_shw_conn.setDisplayName(hot_water.display_name)
    # assign the flow of water
    total_flow = (hot_water.flow_per_area / 3600000.) * room.floor_area
    os_shw_def.setPeakFlowRate(total_flow)
    os_shw_def.setEndUseSubcategory('General')
    # assign schedule
    shw_schedule = os_model.getScheduleByName(hot_water.schedule.identifier)
    if shw_schedule.is_initialized():
        shw_schedule = shw_schedule.get()
        os_shw.setFlowRateFractionSchedule(shw_schedule)
    # assign sensible fraction if it exists
    sens_fract = round(hot_water.sensible_fraction)
    sens_sch_name = '{} Hot Water Sensible Fraction'.format(sens_fract)
    sens_fract_sch = _create_constant_schedule(sens_sch_name, sens_fract, os_model)
    os_shw_def.setSensibleFractionSchedule(sens_fract_sch)
    # assign latent fraction if it exists
    lat_fract = round(hot_water.latent_fraction, 3)
    lat_sch_name = '{} Hot Water Latent Fraction'.format(hot_water.latent_fraction)
    lat_fract_sch = _create_constant_schedule(lat_sch_name, lat_fract, os_model)
    os_shw_def.setLatentFractionSchedule(lat_fract_sch)
    # assign the hot water temperature
    target_temp = round(hot_water.target_temperature, 3)
    target_sch_name = '{}C Hot Water'.format(target_temp)
    target_water_sch = _create_constant_schedule(target_sch_name, target_temp, os_model)
    os_shw_def.setTargetTemperatureSchedule(target_water_sch)
    # create the hot water connection with same temperature as target temperature
    os_shw_conn.setHotWaterSupplyTemperatureSchedule(target_water_sch)
    return os_shw_conn


def infiltration_to_openstudio(infiltration, os_model):
    """Convert Honeybee Infiltration to OpenStudio SpaceInfiltrationDesignFlowRate."""
    # create infiltration OpenStudio object and set identifier
    os_inf = OSSpaceInfiltrationDesignFlowRate(os_model)
    os_inf.setName(infiltration.identifier)
    if infiltration._display_name is not None:
        os_inf.setDisplayName(infiltration.display_name)
    # assign flow per surface
    os_inf.setFlowperExteriorSurfaceArea(infiltration.flow_per_exterior_area)
    # assign schedule
    inf_schedule = os_model.getScheduleByName(infiltration.schedule.identifier)
    if inf_schedule.is_initialized():
        inf_schedule = inf_schedule.get()
        os_inf.setSchedule(inf_schedule)
    # assign constant, temperature, and velocity coefficients
    os_inf.setConstantTermCoefficient(infiltration.constant_coefficient)
    os_inf.setTemperatureTermCoefficient(infiltration.temperature_coefficient)
    os_inf.setVelocityTermCoefficient(infiltration.velocity_coefficient)
    return os_inf


def ventilation_to_openstudio(ventilation, os_model):
    """Convert Honeybee Ventilation to OpenStudio DesignSpecificationOutdoorAir."""
    # create ventilation OpenStudio object and set identifier
    os_vent = OSDesignSpecificationOutdoorAir(os_model)
    os_vent.setName(ventilation.identifier)
    if ventilation._display_name is not None:
        os_vent.setDisplayName(ventilation.display_name)
    # assign air changes per hour if it exists
    os_vent.setOutdoorAirFlowAirChangesperHour(ventilation.air_changes_per_hour)
    os_vent.setOutdoorAirFlowRate(ventilation.flow_per_zone)
    os_vent.setOutdoorAirFlowperPerson(ventilation.flow_per_person)
    os_vent.setOutdoorAirFlowperFloorArea(ventilation.flow_per_area)
    # set the schedule if it exists
    if ventilation.schedule is not None:
        vent_sch = os_model.getScheduleByName(ventilation.schedule.identifier)
        if vent_sch.is_initialized():
            vent_sch = vent_sch.get()
            os_vent.setOutdoorAirFlowRateFractionSchedule(vent_sch)
    return os_vent


def setpoint_to_openstudio_thermostat(setpoint, os_model, zone_identifier=None):
    """Convert Honeybee Setpoint to OpenStudio ThermostatSetpointDualSetpoint."""
    # create thermostat OpenStudio object and set identifier
    os_thermostat = OSThermostatSetpointDualSetpoint(os_model)
    if zone_identifier:
        os_thermostat.setName('{}..{}'.format(setpoint.identifier, zone_identifier))
    else:
        os_thermostat.setName(setpoint.identifier)
    if setpoint._display_name is not None:
        os_thermostat.setDisplayName(setpoint.display_name)
    # assign heating schedule
    heat_schedule = os_model.getScheduleByName(setpoint.heating_schedule.identifier)
    if heat_schedule.is_initialized():
        heat_schedule = heat_schedule.get()
        os_thermostat.setHeatingSetpointTemperatureSchedule(heat_schedule)
    # assign cooling schedule
    cool_schedule = os_model.getScheduleByName(setpoint.cooling_schedule.identifier)
    if cool_schedule.is_initialized():
        cool_schedule = cool_schedule.get()
        os_thermostat.setCoolingSetpointTemperatureSchedule(cool_schedule)
    # assign the setpoint_cutout_difference
    if setpoint.setpoint_cutout_difference != 0:
        os_thermostat.setTemperatureDifferenceBetweenCutoutAndSetpoint(
            setpoint.setpoint_cutout_difference)
    return os_thermostat


def setpoint_to_openstudio_humidistat(setpoint, os_model, zone_identifier=None):
    """Convert Honeybee Setpoint to OpenStudio ZoneControlHumidistat."""
    # create a humidistat if specified
    if setpoint.humidifying_schedule is not None:
        os_humidistat = OSZoneControlHumidistat(os_model)
        if zone_identifier:
            os_humidistat.setName('{}..{}'.format(setpoint.identifier, zone_identifier))
        else:
            os_humidistat.setName(setpoint.identifier)
        if setpoint._display_name is not None:
            os_humidistat.setDisplayName(setpoint.display_name)
        # assign heating schedule
        humid_sch = os_model.getScheduleByName(setpoint.humidifying_schedule.identifier)
        if humid_sch.is_initialized():
            humid_sch = humid_sch.get()
            os_humidistat.setHumidifyingRelativeHumiditySetpointSchedule(humid_sch)
        # assign cooling schedule
        dehumid_sch = os_model.getScheduleByName(setpoint.dehumidifying_schedule.identifier)
        if dehumid_sch.is_initialized():
            dehumid_sch = dehumid_sch.get()
            os_humidistat.setDehumidifyingRelativeHumiditySetpointSchedule(dehumid_sch)
        return os_humidistat


def daylight_to_openstudio(daylight, os_model):
    """Convert Honeybee DaylightingControl to OpenStudio DaylightingControl."""
    # create daylighting control OpenStudio object and set identifier
    os_daylight = OSDaylightingControl(os_model)
    # assign the position of the sensor point
    os_daylight.setPositionXCoordinate(daylight.sensor_position.x)
    os_daylight.setPositionYCoordinate(daylight.sensor_position.y)
    os_daylight.setPositionZCoordinate(daylight.sensor_position.z)
    # set all of the other properties of the daylight control
    os_daylight.setIlluminanceSetpoint(daylight.illuminance_setpoint)
    os_daylight.setMinimumInputPowerFractionforContinuousDimmingControl(
        daylight.min_power_input)
    os_daylight.setMinimumLightOutputFractionforContinuousDimmingControl(
        daylight.min_light_output)
    if daylight.off_at_minimum:
        os_daylight.setLightingControlType('Continuous/Off')
    else:
        os_daylight.setLightingControlType('Continuous')
    return os_daylight


"""____________TRANSLATORS FROM OPENSTUDIO____________"""


def people_from_openstudio(os_people, schedules=None):
    """Convert OpenStudio People object to Honeybee People."""
    # create the people object
    load_def = os_people.peopleDefinition()
    people_per_area = float(str(load_def.peopleperSpaceFloorArea())) \
        if load_def.peopleperSpaceFloorArea().is_initialized() else 0
    occupancy_schedule = always_on
    if schedules is not None and os_people.numberofPeopleSchedule().is_initialized():
        schedule = os_people.numberofPeopleSchedule().get()
        try:
            occupancy_schedule = schedules[schedule.nameString()]
        except KeyError:
            occupancy_schedule = always_on
    people = People(clean_ep_string(os_people.nameString()),
                    people_per_area, occupancy_schedule)
    # assign the optional attributes
    if os_people.activityLevelSchedule().is_initialized():
        schedule = os_people.activityLevelSchedule().get()
        try:
            people.activity_schedule = schedules[schedule.nameString()]
        except KeyError:
            pass
    people.radiant_fraction = load_def.fractionRadiant()
    if not load_def.isSensibleHeatFractionAutocalculated():
        sensible_fraction = load_def.sensibleHeatFraction()
        if sensible_fraction.is_initialized():
            people.latent_fraction = 1 - sensible_fraction.get()
    if os_people.displayName().is_initialized():
        people.display_name = os_people.displayName().get()
    return people


def lighting_from_openstudio(os_lighting, schedules=None):
    """Convert OpenStudio Lights object to Honeybee Lighting."""
    # create the lighting object
    loads_def = os_lighting.lightsDefinition()
    watts_per_area = float(loads_def.wattsperSpaceFloorArea().get()) \
        if loads_def.wattsperSpaceFloorArea().is_initialized() else 0
    schedule = always_on
    if schedules is not None and os_lighting.schedule().is_initialized():
        schedule = os_lighting.schedule().get()
        try:
            schedule = schedules[schedule.nameString()]
        except KeyError:
            schedule = always_on
    lighting = Lighting(clean_ep_string(os_lighting.nameString()),
                        watts_per_area, schedule)
    # assign the optional attributes
    lighting.visible_fraction = float(loads_def.fractionVisible())
    lighting.radiant_fraction = float(loads_def.fractionRadiant())
    lighting.return_air_fraction = float(loads_def.returnAirFraction())
    if os_lighting.displayName().is_initialized():
        lighting.display_name = os_lighting.displayName().get()
    return lighting


def electric_equipment_from_openstudio(os_equipment, schedules=None):
    """Convert OpenStudio ElectricEquipment object to Honeybee ElectricEquipment."""
    # create the equipment object
    loads_def = os_equipment.electricEquipmentDefinition()
    watts_per_area = float(loads_def.wattsperSpaceFloorArea().get()) \
        if loads_def.wattsperSpaceFloorArea().is_initialized() else 0
    schedule = always_on
    if schedules is not None and os_equipment.schedule().is_initialized():
        schedule = os_equipment.schedule().get()
        try:
            schedule = schedules[schedule.nameString()]
        except KeyError:
            schedule = always_on
    equipment = ElectricEquipment(clean_ep_string(os_equipment.nameString()),
                                  watts_per_area, schedule)
    # assign the optional attributes
    equipment.radiant_fraction = float(loads_def.fractionRadiant())
    equipment.latent_fraction = float(loads_def.fractionLatent())
    equipment.lost_fraction = float(loads_def.fractionLost())
    if os_equipment.displayName().is_initialized():
        equipment.display_name = os_equipment.displayName().get()
    return equipment


def gas_equipment_from_openstudio(os_equipment, schedules=None):
    """Convert OpenStudio GasEquipment object to Honeybee GasEquipment."""
    # create the equipment object
    loads_def = os_equipment.gasEquipmentDefinition()
    watts_per_area = float(loads_def.wattsperSpaceFloorArea().get()) \
        if loads_def.wattsperSpaceFloorArea().is_initialized() else 0
    schedule = always_on
    if schedules is not None and os_equipment.schedule().is_initialized():
        schedule = os_equipment.schedule().get()
        try:
            schedule = schedules[schedule.nameString()]
        except KeyError:
            schedule = always_on
    equipment = GasEquipment(clean_ep_string(os_equipment.nameString()),
                             watts_per_area, schedule)
    # assign the optional attributes
    equipment.radiant_fraction = float(loads_def.fractionRadiant())
    equipment.latent_fraction = float(loads_def.fractionLatent())
    equipment.lost_fraction = float(loads_def.fractionLost())
    if os_equipment.displayName().is_initialized():
        equipment.display_name = os_equipment.displayName().get()
    return equipment


def process_from_openstudio(os_process, schedules=None):
    """Convert OpenStudio OtherEquipment object to Honeybee Process."""
    # create the process object
    loads_def = os_process.otherEquipmentDefinition()
    watts = float(loads_def.designLevel().get()) \
        if loads_def.designLevel().is_initialized() else 0
    schedule = always_on
    if schedules is not None and os_process.schedule().is_initialized():
        schedule = os_process.schedule().get()
        try:
            schedule = schedules[schedule.nameString()]
        except KeyError:
            schedule = always_on
    fuel_type = os_process.fuelType()
    equipment = Process(clean_ep_string(os_process.nameString()),
                        watts, schedule, fuel_type)
    # assign the optional attributes
    equipment.end_use_category = os_process.endUseSubcategory()
    equipment.radiant_fraction = float(loads_def.fractionRadiant())
    equipment.latent_fraction = float(loads_def.fractionLatent())
    equipment.lost_fraction = float(loads_def.fractionLost())
    if os_process.displayName().is_initialized():
        equipment.display_name = os_process.displayName().get()
    return equipment


def hot_water_from_openstudio(os_hot_water, floor_area, schedules=None):
    """Convert list of OpenStudio WaterUseEquipment to Honeybee ServiceHotWater."""
    if isinstance(os_hot_water, (list, tuple)) and len(os_hot_water) == 0:
        return None
    elif os_vector_len(os_hot_water) == 0:
        return None
    # sum up all of the flow rates
    flow_per_area, schedule = 0, always_on
    for load in os_hot_water:
        load_def = load.waterUseEquipmentDefinition()
        peak_flow = load_def.peakFlowRate()
        # unit for flow per area is L/h-m2 (m3/s = 3600000 L/h)
        flow_per_area += ((peak_flow * 3600000.) / floor_area)
        if schedules is not None and load.flowRateFractionSchedule().is_initialized():
            schedule = load.flowRateFractionSchedule().get()
            try:
                schedule = schedules[schedule.nameString()]
            except KeyError:
                pass
    hot_water = ServiceHotWater(clean_ep_string(load.nameString()),
                                flow_per_area, schedule)
    # assign the optional attributes
    if load.displayName().is_initialized():
        hot_water.display_name = load.displayName().get()
    return hot_water


def infiltration_from_openstudio(os_infiltration, schedules=None):
    """Convert OpenStudio SpaceInfiltrationDesignFlowRate to Honeybee Infiltration."""
    # create the infiltration object
    flow_per_exterior_area = os_infiltration.flowperExteriorSurfaceArea().get() \
        if os_infiltration.flowperExteriorSurfaceArea().is_initialized() else 0
    schedule = always_on
    if schedules is not None and os_infiltration.schedule().is_initialized():
        try:
            schedule = schedules[os_infiltration.schedule().get().nameString()]
        except KeyError:
            schedule = always_on
    infiltration = Infiltration(clean_ep_string(os_infiltration.nameString()),
                                flow_per_exterior_area, schedule)
    # assign the optional attributes
    infiltration.constant_coefficient = os_infiltration.constantTermCoefficient()
    infiltration.temperature_coefficient = os_infiltration.temperatureTermCoefficient()
    infiltration.velocity_coefficient = os_infiltration.velocityTermCoefficient()
    if os_infiltration.displayName().is_initialized():
        infiltration.display_name = os_infiltration.displayName().get()
    return infiltration


def ventilation_from_openstudio(os_ventilation, schedules=None):
    """Convert OpenStudio DesignSpecificationOutdoorAir to Honeybee Ventilation."""
    # create the ventilation object
    ventilation = Ventilation(clean_ep_string(os_ventilation.nameString()))
    # assign the optional attributes
    ventilation.air_changes_per_hour = os_ventilation.outdoorAirFlowAirChangesperHour()
    ventilation.flow_per_zone = os_ventilation.outdoorAirFlowRate()
    ventilation.flow_per_person = os_ventilation.outdoorAirFlowperPerson()
    ventilation.flow_per_area = os_ventilation.outdoorAirFlowperFloorArea()
    if schedules is not None and \
            os_ventilation.outdoorAirFlowRateFractionSchedule().is_initialized():
        schedule = os_ventilation.outdoorAirFlowRateFractionSchedule().get()
        try:
            ventilation.schedule = schedules[schedule.nameString()]
        except KeyError:
            pass
    if os_ventilation.displayName().is_initialized():
        ventilation.display_name = os_ventilation.displayName().get()
    return ventilation


def setpoint_from_openstudio_thermostat(os_thermostat, schedules=None):
    """Convert OpenStudio ThermostatSetpointDualSetpoint to Honeybee Setpoint."""
    # create the setpoint object
    heat_sch = None
    if os_thermostat.heatingSetpointTemperatureSchedule().is_initialized():
        heat_sch = os_thermostat.heatingSetpointTemperatureSchedule().get()
        try:
            heat_sch = schedules[heat_sch.nameString()]
        except KeyError:
            heat_sch = None
    if heat_sch is None:
        heat_sch = ScheduleRuleset.from_constant_value('No Heating', -100, temperature)
    cool_sch = None
    if os_thermostat.coolingSetpointTemperatureSchedule().is_initialized():
        cool_sch = os_thermostat.coolingSetpointTemperatureSchedule().get()
        try:
            cool_sch = schedules[cool_sch.nameString()]
        except KeyError:
            cool_sch = None
    if cool_sch is None:
        cool_sch = ScheduleRuleset.from_constant_value('No Cooling', 100, temperature)
    setpoint = Setpoint(clean_ep_string(os_thermostat.nameString()), heat_sch, cool_sch)
    # assign the optional attributes
    if os_thermostat.displayName().is_initialized():
        setpoint.display_name = os_thermostat.displayName().get()
    return setpoint


def setpoint_from_openstudio_humidistat(os_humidistat, setpoint, schedules=None):
    """Convert OpenStudio ZoneControlHumidistat to Honeybee Setpoint."""
    # create the setpoint object
    if os_humidistat.humidifyingRelativeHumiditySetpointSchedule().is_initialized():
        humid_sch = os_humidistat.humidifyingRelativeHumiditySetpointSchedule().get()
        try:
            setpoint.humidifying_schedule = schedules[humid_sch.nameString()]
        except KeyError:
            pass
    if os_humidistat.dehumidifyingRelativeHumiditySetpointSchedule().is_initialized():
        dehumid_sch = os_humidistat.dehumidifyingRelativeHumiditySetpointSchedule().get()
        try:
            setpoint.dehumidifying_schedule = schedules[dehumid_sch.nameString()]
        except KeyError:
            pass
    return setpoint


def daylight_from_openstudio(os_daylight):
    """Convert OpenStudio DaylightingControl to Honeybee DaylightingControl."""
    # create the daylight control object
    sensor_position = Point3D(
        os_daylight.positionXCoordinate(), os_daylight.positionYCoordinate(),
        os_daylight.positionZCoordinate())
    daylight = DaylightingControl(sensor_position)
    # assign all of the optional attributes
    daylight.illuminance_setpoint = os_daylight.illuminanceSetpoint()
    daylight.min_power_input = \
        os_daylight.minimumInputPowerFractionforContinuousDimmingControl()
    daylight.min_light_output = \
        os_daylight.minimumLightOutputFractionforContinuousDimmingControl()
    control_type = os_daylight.lightingControlType().lower()
    daylight.off_at_minimum = True if control_type == 'continuous/off' else False
    return daylight



# coding=utf-8
"""OpenStudio material translators."""
from __future__ import division

from honeybee.typing import clean_ep_string
from honeybee_energy.material.opaque import EnergyMaterial, EnergyMaterialNoMass, \
    EnergyMaterialVegetation
from honeybee_energy.material.glazing import EnergyWindowMaterialGlazing, \
    EnergyWindowMaterialSimpleGlazSys
from honeybee_energy.material.gas import EnergyWindowMaterialGas, \
    EnergyWindowMaterialGasMixture, EnergyWindowMaterialGasCustom
from honeybee_energy.material.frame import EnergyWindowFrame
from honeybee_energy.material.shade import EnergyWindowMaterialShade, \
    EnergyWindowMaterialBlind

from honeybee_openstudio.openstudio import OSStandardOpaqueMaterial, \
    OSMasslessOpaqueMaterial, OSRoofVegetation, OSStandardGlazing, OSSimpleGlazing, \
    OSGas, OSGasMixture, OSShade, OSBlind, OSWindowPropertyFrameAndDivider


"""____________TRANSLATORS TO OPENSTUDIO____________"""


def opaque_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyMaterial to OpenStudio StandardOpaqueMaterial."""
    os_opaque_mat = OSStandardOpaqueMaterial(os_model)
    os_opaque_mat.setName(material.identifier)
    if material._display_name is not None:
        os_opaque_mat.setDisplayName(material.display_name)
    os_opaque_mat.setThickness(material.thickness)
    os_opaque_mat.setConductivity(material.conductivity)
    os_opaque_mat.setDensity(material.density)
    os_opaque_mat.setSpecificHeat(material.specific_heat)
    os_opaque_mat.setRoughness(material.roughness)
    os_opaque_mat.setThermalAbsorptance(material.thermal_absorptance)
    os_opaque_mat.setSolarAbsorptance(material.solar_absorptance)
    os_opaque_mat.setVisibleAbsorptance(material.visible_absorptance)
    return os_opaque_mat


def opaque_no_mass_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyMaterialNoMass to OpenStudio MasslessOpaqueMaterial."""
    os_nomass_mat = OSMasslessOpaqueMaterial(os_model)
    os_nomass_mat.setName(material.identifier)
    if material._display_name is not None:
        os_nomass_mat.setDisplayName(material.display_name)
    os_nomass_mat.setThermalResistance(material.r_value)
    os_nomass_mat.setRoughness(material.roughness)
    os_nomass_mat.setThermalAbsorptance(material.thermal_absorptance)
    os_nomass_mat.setSolarAbsorptance(material.solar_absorptance)
    os_nomass_mat.setVisibleAbsorptance(material.visible_absorptance)
    return os_nomass_mat


def vegetation_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyMaterialVegetation to OpenStudio RoofVegetation."""
    os_veg_mat = OSRoofVegetation(os_model)
    os_veg_mat.setName(material.identifier)
    if material._display_name is not None:
        os_veg_mat.setDisplayName(material.display_name)
    os_veg_mat.setThickness(material.thickness)
    os_veg_mat.setConductivityofDrySoil(material.conductivity)
    os_veg_mat.setDensityofDrySoil(material.density)
    os_veg_mat.setSpecificHeatofDrySoil(material.specific_heat)
    os_veg_mat.setRoughness(material.roughness)
    os_veg_mat.setThermalAbsorptance(material.soil_thermal_absorptance)
    os_veg_mat.setSolarAbsorptance(material.soil_solar_absorptance)
    os_veg_mat.setVisibleAbsorptance(material.soil_visible_absorptance)
    os_veg_mat.setHeightofPlants(material.plant_height)
    os_veg_mat.setLeafAreaIndex(material.leaf_area_index)
    os_veg_mat.setLeafReflectivity(material.leaf_reflectivity)
    os_veg_mat.setLeafEmissivity(material.leaf_emissivity)
    os_veg_mat.setMinimumStomatalResistance(material.min_stomatal_resist)
    os_veg_mat.setSaturationVolumetricMoistureContentoftheSoilLayer(material.sat_vol_moist_cont)
    os_veg_mat.setResidualVolumetricMoistureContentoftheSoilLayer(material.residual_vol_moist_cont)
    os_veg_mat.setInitialVolumetricMoistureContentoftheSoilLayer(material.init_vol_moist_cont)
    os_veg_mat.setMoistureDiffusionCalculationMethod(material.moist_diff_model)
    return os_veg_mat


def glazing_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialGlazing to OpenStudio StandardGlazing."""
    os_glazing = OSStandardGlazing(os_model)
    os_glazing.setName(material.identifier)
    if material._display_name is not None:
        os_glazing.setDisplayName(material.display_name)
    os_glazing.setThickness(material.thickness)
    os_glazing.setSolarTransmittanceatNormalIncidence(material.solar_transmittance)
    os_glazing.setFrontSideSolarReflectanceatNormalIncidence(material.solar_reflectance)
    os_glazing.setBackSideSolarReflectanceatNormalIncidence(material.solar_reflectance_back)
    os_glazing.setVisibleTransmittanceatNormalIncidence(material.visible_transmittance)
    os_glazing.setFrontSideVisibleReflectanceatNormalIncidence(material.visible_reflectance)
    os_glazing.setBackSideVisibleReflectanceatNormalIncidence(material.visible_reflectance_back)
    os_glazing.setInfraredTransmittanceatNormalIncidence(material.infrared_transmittance)
    os_glazing.setFrontSideInfraredHemisphericalEmissivity(material.emissivity)
    os_glazing.setBackSideInfraredHemisphericalEmissivity(material.emissivity_back)
    os_glazing.setThermalConductivity(material.conductivity)
    os_glazing.setDirtCorrectionFactorforSolarandVisibleTransmittance(material.dirt_correction)
    os_glazing.setSolarDiffusing(material.solar_diffusing)
    return os_glazing


def simple_glazing_sys_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialSimpleGlazSys to OpenStudio SimpleGlazing."""
    os_glz_sys = OSSimpleGlazing(os_model)
    os_glz_sys.setName(material.identifier)
    if material._display_name is not None:
        os_glz_sys.setDisplayName(material.display_name)
    os_glz_sys.setUFactor(material.u_factor)
    os_glz_sys.setSolarHeatGainCoefficient(material.shgc)
    os_glz_sys.setVisibleTransmittance(material.vt)
    return os_glz_sys


def gas_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialGas to OpenStudio Gas."""
    os_gas = OSGas(os_model)
    os_gas.setName(material.identifier)
    if material._display_name is not None:
        os_gas.setDisplayName(material.display_name)
    os_gas.setThickness(material.thickness)
    os_gas.setGasType(material.gas_type)
    return os_gas


def gas_mixture_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialGasMixture to OpenStudio GasMixture."""
    os_gas_mix = OSGasMixture(os_model)
    os_gas_mix.setName(material.identifier)
    if material._display_name is not None:
        os_gas_mix.setDisplayName(material.display_name)
    os_gas_mix.setThickness(material.thickness)
    for i in range(len(material.gas_types)):
        os_gas_mix.setGasType(i, material.gas_types[i])
        os_gas_mix.setGasFraction(i, material.gas_fractions[i])
    os_gas_mix.setNumberofGasesinMixture(len(material.gas_types))
    return os_gas_mix


def gas_custom_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialGasCustom to OpenStudio Gas."""
    os_gas_custom = OSGas(os_model)
    os_gas_custom.setName(material.identifier)
    if material._display_name is not None:
        os_gas_custom.setDisplayName(material.display_name)
    os_gas_custom.setThickness(material.thickness)
    os_gas_custom.setGasType('Custom')
    os_gas_custom.setConductivityCoefficientA(material.conductivity_coeff_a)
    os_gas_custom.setViscosityCoefficientA(material.viscosity_coeff_a)
    os_gas_custom.setSpecificHeatCoefficientA(material.specific_heat_coeff_a)
    os_gas_custom.setConductivityCoefficientB(material.conductivity_coeff_b)
    os_gas_custom.setViscosityCoefficientB(material.viscosity_coeff_b)
    os_gas_custom.setSpecificHeatCoefficientB(material.specific_heat_coeff_b)
    os_gas_custom.setConductivityCoefficientC(material.conductivity_coeff_c)
    os_gas_custom.setViscosityCoefficientC(material.viscosity_coeff_c)
    os_gas_custom.setSpecificHeatCoefficientC(material.specific_heat_coeff_c)
    os_gas_custom.setSpecificHeatRatio(material.specific_heat_ratio)
    os_gas_custom.setMolecularWeight(material.molecular_weight)
    return os_gas_custom


def shade_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialShade to OpenStudio Shade."""
    os_shade_mat = OSShade(os_model)
    os_shade_mat.setName(material.identifier)
    if material._display_name is not None:
        os_shade_mat.setDisplayName(material.display_name)
    os_shade_mat.setSolarTransmittance(material.solar_transmittance)
    os_shade_mat.setSolarReflectance(material.solar_reflectance)
    os_shade_mat.setVisibleTransmittance(material.visible_transmittance)
    os_shade_mat.setVisibleReflectance(material.visible_reflectance)
    os_shade_mat.setThermalHemisphericalEmissivity(material.emissivity)
    os_shade_mat.setThermalTransmittance(material.infrared_transmittance)
    os_shade_mat.setThickness(material.thickness)
    os_shade_mat.setConductivity(material.conductivity)
    os_shade_mat.setShadetoGlassDistance(material.distance_to_glass)
    os_shade_mat.setTopOpeningMultiplier(material.top_opening_multiplier)
    os_shade_mat.setBottomOpeningMultiplier(material.bottom_opening_multiplier)
    os_shade_mat.setLeftSideOpeningMultiplier(material.left_opening_multiplier)
    os_shade_mat.setRightSideOpeningMultiplier(material.right_opening_multiplier)
    os_shade_mat.setAirflowPermeability(material.airflow_permeability)
    return os_shade_mat


def blind_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowMaterialBlind to OpenStudio Blind."""
    os_blind = OSBlind(os_model)
    os_blind.setName(material.identifier)
    if material._display_name is not None:
        os_blind.setDisplayName(material.display_name)
    os_blind.setSlatOrientation(material.slat_orientation)
    os_blind.setSlatWidth(material.slat_width)
    os_blind.setSlatSeparation(material.slat_separation)
    os_blind.setSlatThickness(material.slat_thickness)
    os_blind.setSlatAngle(material.slat_angle)
    os_blind.setSlatConductivity(material.slat_conductivity)
    os_blind.setSlatBeamSolarTransmittance(material.beam_solar_transmittance)
    os_blind.setFrontSideSlatBeamSolarReflectance(material.beam_solar_reflectance)
    os_blind.setBackSideSlatBeamSolarReflectance(material.beam_solar_reflectance_back)
    os_blind.setSlatDiffuseSolarTransmittance(material.diffuse_solar_transmittance)
    os_blind.setFrontSideSlatDiffuseSolarReflectance(material.diffuse_solar_reflectance)
    os_blind.setBackSideSlatDiffuseSolarReflectance(material.diffuse_solar_reflectance_back)
    os_blind.setSlatDiffuseVisibleTransmittance(material.diffuse_visible_transmittance)
    os_blind.setFrontSideSlatDiffuseVisibleReflectance(material.diffuse_visible_reflectance)
    os_blind.setBackSideSlatDiffuseVisibleReflectance(material.diffuse_visible_reflectance_back)
    os_blind.setSlatBeamVisibleTransmittance(material.beam_visible_transmittance)
    os_blind.setFrontSideSlatBeamVisibleReflectance(material.beam_visible_reflectance)
    os_blind.setBackSideSlatBeamVisibleReflectance(material.beam_visible_reflectance_back)
    os_blind.setSlatInfraredHemisphericalTransmittance(material.infrared_transmittance)
    os_blind.setFrontSideSlatInfraredHemisphericalEmissivity(material.emissivity)
    os_blind.setBackSideSlatInfraredHemisphericalEmissivity(material.emissivity_back)
    os_blind.setBlindtoGlassDistance(material.distance_to_glass)
    os_blind.setBlindTopOpeningMultiplier(material.top_opening_multiplier)
    os_blind.setBlindBottomOpeningMultiplier(material.bottom_opening_multiplier)
    os_blind.setBlindLeftSideOpeningMultiplier(material.left_opening_multiplier)
    os_blind.setBlindRightSideOpeningMultiplier(material.right_opening_multiplier)
    return os_blind


def frame_material_to_openstudio(material, os_model):
    """Convert Honeybee EnergyWindowFrame to OpenStudio WindowPropertyFrameAndDivider."""
    os_frame_mat = OSWindowPropertyFrameAndDivider(os_model)
    os_frame_mat.setName(material.identifier)
    if material._display_name is not None:
        os_frame_mat.setDisplayName(material.display_name)
    os_frame_mat.setFrameWidth(material.width)
    os_frame_mat.setFrameConductance(material.conductance)
    os_frame_mat.setRatioOfFrameEdgeGlassConductanceToCenterOfGlassConductance(
        material.edge_to_center_ratio)
    os_frame_mat.setFrameOutsideProjection(material.outside_projection)
    os_frame_mat.setFrameInsideProjection(material.inside_projection)
    os_frame_mat.setFrameThermalHemisphericalEmissivity(material.thermal_absorptance)
    os_frame_mat.setFrameSolarAbsorptance(material.solar_absorptance)
    os_frame_mat.setFrameVisibleAbsorptance(material.visible_absorptance)
    return os_frame_mat


def material_to_openstudio(material, os_model):
    """Convert any Honeybee energy material into an OpenStudio object.

    Args:
        material: A honeybee-energy Python object of a material layer.
        os_model: The OpenStudio Model object to which the Room will be added.

    Returns:
        An OpenStudio object for the material.
    """
    if isinstance(material, EnergyMaterial):
        return opaque_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyMaterialNoMass):
        return opaque_no_mass_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyMaterialVegetation):
        return vegetation_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialGlazing):
        return glazing_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialSimpleGlazSys):
        return simple_glazing_sys_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialGas):
        return gas_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialGasMixture):
        return gas_mixture_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialGasCustom):
        return gas_custom_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowFrame):
        return frame_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialShade):
        return shade_material_to_openstudio(material, os_model)
    elif isinstance(material, EnergyWindowMaterialBlind):
        return blind_material_to_openstudio(material, os_model)
    else:
        raise ValueError(
            '{} is not a recognized Energy Material type'.format(type(material)))


"""____________TRANSLATORS FROM OPENSTUDIO____________"""


def opaque_material_from_openstudio(os_material):
    """Convert OpenStudio StandardOpaqueMaterial to Honeybee EnergyMaterial."""
    # create the material object
    thickness = os_material.thickness()
    conductivity = os_material.conductivity()
    density = os_material.density()
    specific_heat = os_material.specificHeat()
    material = EnergyMaterial(clean_ep_string(os_material.nameString()),
                              thickness, conductivity, density, specific_heat)
    # set the optional properties of the material
    _apply_roughness(os_material, material)
    material.thermal_absorptance = os_material.thermalAbsorptance()
    material.solar_absorptance = os_material.solarAbsorptance()
    material.visible_absorptance = os_material.visibleAbsorptance()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def opaque_no_mass_material_from_openstudio(os_material):
    """Convert OpenStudio MasslessOpaqueMaterial to Honeybee EnergyMaterialNoMass."""
    # create the material object
    r_value = os_material.thermalResistance()
    material = EnergyMaterialNoMass(clean_ep_string(os_material.nameString()), r_value)
    # set the optional properties of the material
    try:
        _apply_roughness(os_material, material)
    except AttributeError:
        return material  # OpenStudio AirGap material with no roughness
    if os_material.thermalAbsorptance().is_initialized():
        material.thermal_absorptance = os_material.thermalAbsorptance().get()
    if os_material.solarAbsorptance().is_initialized():
        material.solar_absorptance = os_material.solarAbsorptance().get()
    if os_material.visibleAbsorptance().is_initialized():
        material.visible_absorptance = os_material.visibleAbsorptance().get()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def vegetation_material_from_openstudio(os_material):
    """Convert OpenStudio RoofVegetation to Honeybee EnergyMaterialVegetation."""
    # create the material object
    thickness = os_material.thickness()
    conductivity = os_material.conductivityofDrySoil()
    density = os_material.densityofDrySoil()
    specific_heat = os_material.specificHeatofDrySoil()
    plant_height = os_material.heightofPlants()
    leaf_area_index = os_material.leafAreaIndex()
    leaf_reflectivity = os_material.leafReflectivity()
    leaf_emissivity = os_material.leafEmissivity()
    min_stomatal_resist = os_material.minimumStomatalResistance()
    material = EnergyMaterialVegetation(
        clean_ep_string(os_material.nameString()),
        thickness, conductivity, density, specific_heat,
        plant_height=plant_height, leaf_area_index=leaf_area_index,
        leaf_reflectivity=leaf_reflectivity, leaf_emissivity=leaf_emissivity,
        min_stomatal_resist=min_stomatal_resist)
    # set the other required properties
    material.sat_vol_moist_cont = os_material.saturationVolumetricMoistureContent()
    material.residual_vol_moist_cont = os_material.residualVolumetricMoistureContent()
    material.init_vol_moist_cont = os_material.initialVolumetricMoistureContent()
    material.moist_diff_model = os_material.moistureDiffusionCalculationMethod().title()
    # set the optional properties of the material
    _apply_roughness(os_material, material)
    if os_material.thermalAbsorptance().is_initialized():
        material.soil_thermal_absorptance = os_material.thermalAbsorptance().get()
    if os_material.solarAbsorptance().is_initialized():
        material.soil_solar_absorptance = os_material.solarAbsorptance().get()
    if os_material.visibleAbsorptance().is_initialized():
        material.soil_visible_absorptance = os_material.visibleAbsorptance().get()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def _apply_roughness(os_material, material):
    """Apply roughness from an OpenStudio Material to a Honeybee Material."""
    rough = os_material.roughness().lower()
    if rough == 'veryrough':
        material.roughness == 'VeryRough'
    elif rough == 'mediumrough':
        material.roughness == 'MediumRough'
    elif rough == 'mediumsmooth':
        material.roughness == 'MediumSmooth'
    elif rough == 'verysmooth':
        material.roughness == 'VerySmooth'
    else:  # Rough or Smooth
        material.roughness = rough.title()


def glazing_material_from_openstudio(os_material):
    """Convert OpenStudio StandardGlazing to Honeybee EnergyWindowMaterialGlazing."""
    if os_material.opticalDataType().lower() == 'spectral':
        return None
    # get the solar transmittance
    try:
        solar_transmittance = os_material.solarTransmittance()
    except Exception:  # spectral material to ignore
        return None
    # create the material objects
    thickness = os_material.thickness()
    conductivity = os_material.conductivity()
    infrared_transmittance = os_material.infraredTransmittance()
    emissivity = os_material.frontSideInfraredHemisphericalEmissivity()
    emissivity_back = os_material.backSideInfraredHemisphericalEmissivity()
    conductivity = os_material.thermalConductivity()
    material = EnergyWindowMaterialGlazing(
        clean_ep_string(os_material.nameString()), thickness, solar_transmittance,
        conductivity=conductivity, infrared_transmittance=infrared_transmittance,
        emissivity=emissivity, emissivity_back=emissivity_back)
    # set all of the optional properties
    if os_material.frontSideSolarReflectanceatNormalIncidence().is_initialized():
        material.solar_reflectance = \
            os_material.frontSideSolarReflectanceatNormalIncidence().get()
    if os_material.backSideSolarReflectanceatNormalIncidence().is_initialized():
        material.solar_reflectance_back = \
            os_material.backSideSolarReflectanceatNormalIncidence().get()
    if os_material.visibleTransmittanceatNormalIncidence().is_initialized():
        material.visible_transmittance = \
            os_material.visibleTransmittanceatNormalIncidence().get()
    if os_material.frontSideVisibleReflectanceatNormalIncidence().is_initialized():
        try:
            material.visible_reflectance = \
                os_material.frontSideVisibleReflectanceatNormalIncidence().get()
        except AssertionError:
            pass  # illegal combination of transmittance and reflectance
    if os_material.backSideVisibleReflectanceatNormalIncidence().is_initialized():
        try:
            material.visible_reflectance_back = \
                os_material.backSideVisibleReflectanceatNormalIncidence().get()
        except AssertionError:
            pass  # illegal combination of transmittance and reflectance
    material.dirt_correction = \
        os_material.dirtCorrectionFactorforSolarandVisibleTransmittance()
    material.solar_diffusing = os_material.solarDiffusing()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def simple_glazing_sys_material_from_openstudio(os_material):
    """Convert OpenStudio SimpleGlazing to Honeybee EnergyWindowMaterialSimpleGlazSys."""
    # create the material object
    u_factor = os_material.uFactor()
    shgc = os_material.solarHeatGainCoefficient()
    material = EnergyWindowMaterialSimpleGlazSys(
        clean_ep_string(os_material.nameString()), u_factor, shgc)
    # set the optional properties of the material
    if os_material.visibleTransmittance().is_initialized():
        material.vt = os_material.visibleTransmittance().get()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def gas_material_from_openstudio(os_material):
    """Convert OpenStudio Gas to Honeybee EnergyWindowMaterialGas."""
    # create the material object
    thickness = os_material.thickness()
    gas_type = os_material.gasType()
    material = EnergyWindowMaterialGas(
        clean_ep_string(os_material.nameString()), thickness, gas_type)
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def gas_mixture_material_from_openstudio(os_material):
    """Convert OpenStudio GasMixture to Honeybee EnergyWindowMaterialGasMixture."""
    thickness = os_material.thickness()
    gas_count = os_material.numberofGasesinMixture()
    gas_types = [os_material.gas1Type(), os_material.gas2Type()]
    gas_fractions = [os_material.gas1Fraction(), os_material.gas2Fraction()]
    if gas_count > 2:
        if os_material.gas3Fraction().is_initialized():
            gas_types.append(os_material.gas3Type())
            gas_fractions.append(os_material.gas3Fraction().get())
        if gas_count > 3:
            if os_material.gas4Fraction().is_initialized():
                gas_types.append(os_material.gas4Type())
                gas_fractions.append(os_material.gas4Fraction().get())
    material = EnergyWindowMaterialGasMixture(
        clean_ep_string(os_material.nameString()), thickness, gas_types, gas_fractions)
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def gas_custom_material_from_openstudio(os_material):
    """Convert OpenStudio Gas to Honeybee EnergyWindowMaterialGasCustom."""
    # create the material object
    thickness = os_material.thickness()
    conductivity_coeff_a, viscosity_coeff_a, specific_heat_coeff_a = 0, 0, 0
    if os_material.customConductivityCoefficientA().is_initialized():
        conductivity_coeff_a = os_material.customConductivityCoefficientA().get()
    if os_material.viscosityCoefficientA().is_initialized():
        viscosity_coeff_a = os_material.viscosityCoefficientA().get()
    if os_material.specificHeatCoefficientA().is_initialized():
        specific_heat_coeff_a = os_material.specificHeatCoefficientA().get()
    material = EnergyWindowMaterialGasCustom(
        clean_ep_string(os_material.nameString()), thickness,
        conductivity_coeff_a, viscosity_coeff_a, specific_heat_coeff_a)
    # set the optional properties of the material
    if os_material.customConductivityCoefficientB().is_initialized():
        material.conductivity_coeff_b = os_material.customConductivityCoefficientB().get()
    if os_material.viscosityCoefficientB().is_initialized():
        material.viscosity_coeff_b = os_material.viscosityCoefficientB().get()
    if os_material.specificHeatCoefficientB().is_initialized():
        material.specific_heat_coeff_b = os_material.specificHeatCoefficientB().get()
    if os_material.customConductivityCoefficientC().is_initialized():
        material.conductivity_coeff_c = os_material.customConductivityCoefficientC().get()
    if os_material.viscosityCoefficientC().is_initialized():
        material.viscosity_coeff_c = os_material.viscosityCoefficientC().get()
    if os_material.specificHeatCoefficientC().is_initialized():
        material.specific_heat_coeff_c = os_material.specificHeatCoefficientC().get()
    if os_material.specificHeatRatio().is_initialized():
        material.specific_heat_ratio = os_material.specificHeatRatio().get()
    if os_material.molecularWeight().is_initialized():
        material.molecular_weight = os_material.molecularWeight().get()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def shade_material_from_openstudio(os_material):
    """Convert OpenStudio Shade to Honeybee EnergyWindowMaterialShade."""
    # create the material object
    material = EnergyWindowMaterialShade(clean_ep_string(os_material.nameString()))
    # set the optional properties of the material
    material.solar_transmittance = os_material.solarTransmittance()
    material.solar_reflectance = os_material.solarReflectance()
    material.visible_transmittance = os_material.visibleTransmittance()
    material.visible_reflectance = os_material.visibleReflectance()
    material.emissivity = os_material.thermalHemisphericalEmissivity()
    material.infrared_transmittance = os_material.thermalTransmittance()
    material.thickness = os_material.thickness()
    material.conductivity = os_material.conductivity()
    material.distance_to_glass = os_material.shadetoGlassDistance()
    material.top_opening_multiplier = os_material.topOpeningMultiplier()
    material.bottom_opening_multiplier = os_material.bottomOpeningMultiplier()
    material.left_opening_multiplier = os_material.leftSideOpeningMultiplier()
    material.right_opening_multiplier = os_material.rightSideOpeningMultiplier()
    material.airflow_permeability = os_material.airflowPermeability()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def blind_material_from_openstudio(os_material):
    """Convert OpenStudio Blind to Honeybee EnergyWindowMaterialBlind."""
    # create the material object
    material = EnergyWindowMaterialBlind(clean_ep_string(os_material.nameString()))
    # set the optional properties of the material
    material.slat_orientation = os_material.slatOrientation()
    material.slat_width = os_material.slatWidth()
    material.slat_separation = os_material.slatSeparation()
    material.slat_thickness = os_material.slatThickness()
    material.slat_width = os_material.slatWidth()
    material.slat_angle = os_material.slatAngle()
    material.slat_conductivity = os_material.slatConductivity()
    material.beam_solar_transmittance = os_material.slatBeamSolarTransmittance()
    material.beam_solar_reflectance = os_material.frontSideSlatBeamSolarReflectance()
    material.beam_solar_reflectance_back = os_material.backSideSlatBeamSolarReflectance()
    material.diffuse_solar_reflectance = \
        os_material.frontSideSlatDiffuseSolarReflectance()
    material.diffuse_solar_reflectance_back = \
        os_material.backSideSlatDiffuseSolarReflectance()
    material.diffuse_visible_transmittance = \
        os_material.slatDiffuseVisibleTransmittance()
    material.infrared_transmittance = \
        os_material.slatInfraredHemisphericalTransmittance()
    material.emissivity = os_material.frontSideSlatInfraredHemisphericalEmissivity()
    material.emissivity_back = os_material.backSideSlatInfraredHemisphericalEmissivity()
    material.distance_to_glass = os_material.blindtoGlassDistance()
    material.top_opening_multiplier = os_material.blindTopOpeningMultiplier()
    material.bottom_opening_multiplier = os_material.blindBottomOpeningMultiplier()
    material.left_opening_multiplier = os_material.blindLeftSideOpeningMultiplier()
    material.right_opening_multiplier = os_material.blindRightSideOpeningMultiplier()
    if os_material.frontSideSlatDiffuseVisibleReflectance().is_initialized():
        material.diffuse_visible_reflectance = \
            os_material.frontSideSlatDiffuseVisibleReflectance().get()
    if os_material.backSideSlatDiffuseVisibleReflectance().is_initialized():
        material.diffuse_visible_reflectance_back = \
            os_material.backSideSlatDiffuseVisibleReflectance().get()
    if os_material.displayName().is_initialized():
        material.display_name = os_material.displayName().get()
    return material


def extract_all_materials(os_model):
    """Extract all material objects from an OpenStudio Model.

    Args:
        os_model: The OpenStudio Model object from which materials will be extracted.

    Returns:
        A dictionary of material objects with material identifiers as keys and
        material objects as values.
    """
    materials = {}
    for os_mat in os_model.getStandardOpaqueMaterials():
        mat = opaque_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getMasslessOpaqueMaterials():
        mat = opaque_no_mass_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getAirGaps():
        mat = opaque_no_mass_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getRoofVegetations():
        mat = vegetation_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getSimpleGlazings():
        mat = simple_glazing_sys_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getStandardGlazings():
        mat = glazing_material_from_openstudio(os_mat)
        if mat is not None:
            materials[mat.identifier] = mat
    for os_mat in os_model.getGass():
        if os_mat.gasType() == 'Custom':
            mat = gas_custom_material_from_openstudio(os_mat)
        else:
            mat = gas_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getGasMixtures():
        mat = gas_mixture_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getShades():
        mat = shade_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    for os_mat in os_model.getBlinds():
        mat = blind_material_from_openstudio(os_mat)
        materials[mat.identifier] = mat
    return materials



# coding=utf-8
"""Module taken from OpenStudio-standards.

https://github.com/NREL/openstudio-standards/blob/master/
lib/openstudio-standards/schedules/create.rb
"""
from __future__ import division

from honeybee_openstudio.openstudio import openstudio, openstudio_model, os_vector_len


def create_constant_schedule_ruleset(model, value, name=None, schedule_type_limit=None):
    """Create constant ScheduleRuleset with a given value.

    Args:
        model: [OpenStudio::Model::Model] OpenStudio model object
        value: [Double] the value to use, 24-7, 365
        name: [String] the name of the schedule
        schedule_type_limit: [String] the name of a schedule type limit.
            options are Dimensionless, Temperature, Humidity Ratio, Fraction,
            Fractional, OnOff, and Activity
    """
    # check to see if schedule exists with same name and constant value and return if true
    if name is not None:
        existing_sch = model.getScheduleRulesetByName(name)
        if existing_sch.is_initialized():
            existing_sch = existing_sch.get()
            existing_day_sch_vals = existing_sch.defaultDaySchedule().values()
            if os_vector_len(existing_day_sch_vals) == 1:
                if abs(existing_day_sch_vals[0] - value) < 1.0e-6:
                    return existing_sch
    # create ScheduleRuleset
    schedule = openstudio_model.ScheduleRuleset(model)
    schedule.defaultDaySchedule().addValue(openstudio.Time(0, 24, 0, 0), value)
    # set name
    if name is not None:
        schedule.setName(name)
        schedule.defaultDaySchedule().setName('{} Default'.format(name))
    # set schedule type limits
    if schedule_type_limit is not None:
        sch_type_limits_obj = create_schedule_type_limits(
            model, standard_schedule_type_limit=schedule_type_limit)
        schedule.setScheduleTypeLimits(sch_type_limits_obj)
    return schedule


def create_schedule_type_limits(
        model, standard_schedule_type_limit=None, name=None, lower_limit_value=None,
        upper_limit_value=None, numeric_type=None, unit_type=None):
    """Create a ScheduleTypeLimits object for a schedule.

    Args:
        model: [OpenStudio::Model::Model] OpenStudio model object.
        standard_schedule_type_limit: [String] the name of a standard schedule
            type limit with predefined limits. Options are Dimensionless, Temperature,
            Humidity Ratio, Fraction, Fractional, OnOff, and Activity.
        name: [String] the name of the schedule type limits.
        lower_limit_value: [double] the lower limit value for the schedule type.
        upper_limit_value: [double] the upper limit value for the schedule type.
        numeric_type: [String] the numeric type, options are Continuous or Discrete.
        unit_type: [String] the unit type, options are defined in EnergyPlus I/O reference.
    """

    if standard_schedule_type_limit is None:
        if lower_limit_value is None or upper_limit_value is None or \
                numeric_type is None or unit_type is None:
            msg = 'If calling create_schedule_type_limits without a ' \
                'standard_schedule_type_limit, you must specify all properties ' \
                'of ScheduleTypeLimits.'
            print(msg)
            return None

        schedule_type_limits = openstudio_model.scheduleTypeLimits(model)
        if name is not None:
            schedule_type_limits.setName(name)
        if lower_limit_value is not None:
            schedule_type_limits.setLowerLimitValue(lower_limit_value)
        if upper_limit_value is not None:
            schedule_type_limits.setUpperLimitValue(upper_limit_value)
        if numeric_type is not None:
            schedule_type_limits.setNumericType(numeric_type)
        if unit_type is not None:
            schedule_type_limits.setUnitType(unit_type)
    else:
        schedule_type_limits = model.getScheduleTypeLimitsByName(
            standard_schedule_type_limit)
        if not schedule_type_limits.is_initialized():
            stl_name = standard_schedule_type_limit.lower()
            if stl_name == 'dimensionless':
                schedule_type_limits = openstudio_model.ScheduleTypeLimits(model)
                schedule_type_limits.setName('Dimensionless')
                schedule_type_limits.setLowerLimitValue(0.0)
                schedule_type_limits.setUpperLimitValue(1000.0)
                schedule_type_limits.setNumericType('Continuous')
                schedule_type_limits.setUnitType('Dimensionless')
            elif stl_name == 'temperature':
                schedule_type_limits = openstudio_model.ScheduleTypeLimits(model)
                schedule_type_limits.setName('Temperature')
                schedule_type_limits.setLowerLimitValue(0.0)
                schedule_type_limits.setUpperLimitValue(100.0)
                schedule_type_limits.setNumericType('Continuous')
                schedule_type_limits.setUnitType('Temperature')
            elif stl_name == 'humidity ratio':
                schedule_type_limits = openstudio_model.ScheduleTypeLimits(model)
                schedule_type_limits.setName('Humidity Ratio')
                schedule_type_limits.setLowerLimitValue(0.0)
                schedule_type_limits.setUpperLimitValue(0.3)
                schedule_type_limits.setNumericType('Continuous')
                schedule_type_limits.setUnitType('Dimensionless')
            elif stl_name in ('fraction', 'fractional'):
                schedule_type_limits = openstudio_model.ScheduleTypeLimits(model)
                schedule_type_limits.setName('Fraction')
                schedule_type_limits.setLowerLimitValue(0.0)
                schedule_type_limits.setUpperLimitValue(1.0)
                schedule_type_limits.setNumericType('Continuous')
                schedule_type_limits.setUnitType('Dimensionless')
            elif stl_name == 'onoff':
                schedule_type_limits = openstudio_model.ScheduleTypeLimits(model)
                schedule_type_limits.setName('OnOff')
                schedule_type_limits.setLowerLimitValue(0)
                schedule_type_limits.setUpperLimitValue(1)
                schedule_type_limits.setNumericType('Discrete')
                schedule_type_limits.setUnitType('Availability')
            elif stl_name == 'activity':
                schedule_type_limits = openstudio_model.ScheduleTypeLimits(model)
                schedule_type_limits.setName('Activity')
                schedule_type_limits.setLowerLimitValue(70.0)
                schedule_type_limits.setUpperLimitValue(1000.0)
                schedule_type_limits.setNumericType('Continuous')
                schedule_type_limits.setUnitType('ActivityLevel')
            else:
                msg = 'Invalid standard_schedule_type_limit for method ' \
                    'create_schedule_type_limits.'
                raise ValueError(msg)
        else:
            schedule_type_limits = schedule_type_limits.get()
    return schedule_type_limits


def model_add_schedule(model, schedule_name):
    """Get a schedule from the Model or always on schedule if it does not exist."""
    if schedule_name is None or schedule_name == '':
        return model.alwaysOnDiscreteSchedule()
    schedule = model.getScheduleByName(schedule_name)
    if schedule.is_initialized():
        return schedule.get()
    return model.alwaysOnDiscreteSchedule()
